@@ -9,7 +9,11 @@
 #' @param forest The trained forest.
 #' @param calibrate.weights Whether to force debiasing weights to match expected
 #'                          moments for 1, W, W.hat, and 1/Var[W|X].
-#' 
+#' @param subset Specifies a subset of the training examples over which we
+#'               estimate the ATE. WARNING: For valid statistical performance,
+#'               the subset should be defined only using features Xi, not using
+#'               the treatment Wi or the outcome Yi.
+#'
 #' @examples \dontrun{
 #' n = 2000; p = 10
 #' X = matrix(rnorm(n*p), n, p)
@@ -18,34 +22,60 @@
 #' tau.forest = causal_forest(X, Y, W)
 #' tau.hat = predict(tau.forest)
 #' average_partial_effect(tau.forest)
+#' average_partial_effect(tau.forest, subset = X[,1] > 0)
 #' }
 #'
 #' @return An estimate of the average partial effect, along with standard error.
 #' @export
-average_partial_effect = function(forest, calibrate.weights = TRUE) {
+average_partial_effect = function(forest, calibrate.weights = TRUE, subset = NULL) {
   
   if (!("causal_forest" %in% class(forest))) {
     stop("Average effect estimation only implemented for causal_forest")
   }
+
+  if (is.null(subset)) {
+    subset <- 1:length(forest$Y.hat)
+  }
+  
+  if (class(subset) == "logical" & length(subset) == length(forest$Y.hat)) {
+    subset <- which(subset)
+  }
+  
+  if (!all(subset %in% 1:length(forest$Y.hat))) {
+    stop(paste("If specified, subset must be a vector contained in 1:n,",
+               "or a boolean vector of length n."))
+  }
+
+  # Only use data selected via subsetting.
+  subset.W.orig <- forest$W.orig[subset]
+  subset.W.hat <- forest$W.hat[subset]
+  subset.Y.orig <- forest$Y.orig[subset]
+  subset.Y.hat <- forest$Y.hat[subset]
+  tau.hat <- predict(forest)$predictions[subset]
+  if (length(forest$clusters) == 0) {
+    subset.clusters <- numeric(0)
+  } else {
+    subset.clusters <- forest$clusters[subset]
+  }
   
   # This is a simple plugin estimate of the APE.
-  tau.hat = predict(forest)$predictions
-  cape.plugin = mean(tau.hat)
+  cape.plugin <- mean(tau.hat)
   
   # Estimate the variance of W given X. For binary treatments,
   # we get a good implicit estimator V.hat = e.hat (1 - e.hat), and
   # so this step is not needed. Note that if we use the present CAPE estimator
   # with a binary treatment and set V.hat = e.hat (1 - e.hat), then we recover
   # exactly the AIPW estimator of the CATE.
-  variance_forest <- regression_forest(forest$X.orig, (forest$W.orig - forest$W.hat)^2)
+  variance_forest <- regression_forest(forest$X.orig[subset,],
+                                       (subset.W.orig - subset.W.hat)^2)
   V.hat <- predict(variance_forest)$predictions
-  debiasing.weights = (forest$W.orig - forest$W.hat) / V.hat
+  debiasing.weights <- (subset.W.orig - subset.W.hat) / V.hat
   
   # In the population, we want A' %*% weights = b.
   # Modify debiasing weights gamma to make this true, i.e., compute
   # argmin {||gamma - gamma.original||_2^2 : A'gamma = b}
   if (calibrate.weights) {
-    A = cbind(1, forest$W.orig, forest$W.hat) / length(forest$W.orig)
+    A = cbind(1, subset.W.orig, subset.W.hat) / length(subset.W.orig)
     b = c(0, 1, 0)
     bias = t(A) %*% debiasing.weights - b
     lambda = solve(t(A) %*% A, bias)
@@ -54,20 +84,20 @@ average_partial_effect = function(forest, calibrate.weights = TRUE) {
   }
   
   # Compute a residual-based correction to the plugin.
-  plugin.prediction = forest$Y.hat + (forest$W.orig - forest$W.hat) * tau.hat
-  plugin.residual = forest$Y.orig - plugin.prediction
+  plugin.prediction = subset.Y.hat + (subset.W.orig - subset.W.hat) * tau.hat
+  plugin.residual = subset.Y.orig - plugin.prediction
   cape.correction = mean(debiasing.weights * plugin.residual)
   cape.estimate = cape.plugin + cape.correction
   
   # Estimate variance using the calibration
-  if (length(forest$clusters) == 0) {
-    cape.se = sqrt(mean((debiasing.weights * plugin.residual)^2) / length(forest$W.orig))
+  if (length(subset.clusters) == 0) {
+    cape.se = sqrt(mean((debiasing.weights * plugin.residual)^2) / length(subset.W.orig))
   } else {
     debiasing.clust = Matrix::sparse.model.matrix(
-      ~ factor(forest$clusters) + 0,
+      ~ factor(subset.clusters) + 0,
       transpose = TRUE) %*% (debiasing.weights * plugin.residual)
-    cape.se = sqrt(sum(debiasing.clust^2) / length(forest$W.orig) /
-      (length(forest$W.orig) - 1))
+    cape.se = sqrt(sum(debiasing.clust^2) / length(subset.W.orig) /
+                     (length(subset.W.orig) - 1))
   }
   
   return(c(estimate=cape.estimate, std.err=cape.se))
