@@ -19,8 +19,8 @@
 #' @param Z.hat Estimates of the instrument propensities E[Z | Xi]. If Z.hat = NULL,
 #'              these are estimated using a separate regression forest.
 #' @param sample.fraction Fraction of the data used to build each tree.
-#'                        Note: If honesty is used, these subsamples will
-#'                        further be cut in half.
+#'                        Note: If honesty = TRUE, these subsamples will
+#'                        further be cut by a factor of honesty.fraction.
 #' @param mtry Number of variables tried for each split.
 #' @param num.trees Number of trees grown in the forest. Note: Getting accurate
 #'                  confidence intervals generally requires more trees than
@@ -29,7 +29,10 @@
 #'                    automatically selects an appropriate amount.
 #' @param min.node.size A target for the minimum number of observations in each tree leaf. Note that nodes
 #'                      with size smaller than min.node.size can occur, as in the original randomForest package.
-#' @param honesty Whether or not honest splitting (i.e., sub-sample splitting) should be used.
+#' @param honesty Whether to use honest splitting (i.e., sub-sample splitting).
+#' @param honesty.fraction The fraction of data that will be used for determining splits if honesty = TRUE. Corresponds 
+#'                         to set J1 in the notation of the paper. When using the defaults (honesty = TRUE and 
+#'                         honesty.fraction = NULL), half of the data will be used for determining splits
 #' @param ci.group.size The forst will grow ci.group.size trees on each subsample.
 #'                      In order to provide confidence intervals, ci.group.size must
 #'                      be at least 2.
@@ -40,6 +43,7 @@
 #' @param imbalance.penalty A tuning parameter that controls how harshly imbalanced splits are penalized.
 #' @param stabilize.splits Whether or not the instrument should be taken into account when
 #'                         determining the imbalance of a split (experimental).
+#' @param compute.oob.predictions Whether OOB predictions on training set should be precomputed.
 #' @param seed The seed for the C++ random number generator.
 #' @param clusters Vector of integers or factors specifying which cluster each observation corresponds to.
 #' @param samples_per_cluster If sampling by cluster, the number of observations to be sampled from
@@ -58,11 +62,13 @@ instrumental_forest <- function(X, Y, W, Z,
                                 num.threads = NULL,
                                 min.node.size = NULL,
                                 honesty = TRUE,
+                                honesty.fraction = NULL,
                                 ci.group.size = 2,
                                 reduced.form.weight = 0,
                                 alpha = 0.05,
                                 imbalance.penalty = 0.0,
                                 stabilize.splits = TRUE,
+                                compute.oob.predictions = TRUE,
                                 seed = NULL,
                                 clusters = NULL,
                                 samples_per_cluster = NULL) {
@@ -71,6 +77,7 @@ instrumental_forest <- function(X, Y, W, Z,
     if(length(W) != nrow(X)) { stop("W has incorrect length.") }
     if(length(Z) != nrow(X)) { stop("Z has incorrect length.") }
     
+    
     mtry <- validate_mtry(mtry, X)
     num.threads <- validate_num_threads(num.threads)
     min.node.size <- validate_min_node_size(min.node.size)
@@ -78,6 +85,7 @@ instrumental_forest <- function(X, Y, W, Z,
     seed <- validate_seed(seed)
     clusters <- validate_clusters(clusters, X)
     samples_per_cluster <- validate_samples_per_cluster(samples_per_cluster, clusters)
+    honesty.fraction <- validate_honesty_fraction(honesty.fraction, honesty)
     
     if (!is.numeric(reduced.form.weight) | reduced.form.weight < 0 | reduced.form.weight > 1) {
         stop("Error: Invalid value for reduced.form.weight. Please give a value in [0,1].")
@@ -85,8 +93,8 @@ instrumental_forest <- function(X, Y, W, Z,
     
     if (is.null(Y.hat)) {
       forest.Y <- regression_forest(X, Y, sample.fraction = sample.fraction, mtry = mtry, 
-                                    num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, 
-                                    honesty = TRUE, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
+                                    num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, honesty = TRUE,
+                                    honesty.fraction = NULL, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
                                     clusters = clusters, samples_per_cluster = samples_per_cluster);
       Y.hat <- predict(forest.Y)$predictions
     } else if (length(Y.hat) == 1) {
@@ -97,8 +105,8 @@ instrumental_forest <- function(X, Y, W, Z,
     
     if (is.null(W.hat)) {
       forest.W <- regression_forest(X, W, sample.fraction = sample.fraction, mtry = mtry, 
-                                    num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, 
-                                    honesty = TRUE, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
+                                    num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, honesty = TRUE,
+                                    honesty.fraction = NULL, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
                                     clusters = clusters, samples_per_cluster = samples_per_cluster);
       W.hat <- predict(forest.W)$predictions
     } else if (length(W.hat) == 1) {
@@ -109,8 +117,8 @@ instrumental_forest <- function(X, Y, W, Z,
     
     if (is.null(Z.hat)) {
       forest.Z <- regression_forest(X, Z, sample.fraction = sample.fraction, mtry = mtry, 
-                                    num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, 
-                                    honesty = TRUE, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
+                                    num.trees = min(500, num.trees), num.threads = num.threads, min.node.size = NULL, honesty = TRUE,
+                                    honesty.fraction = NULL, seed = seed, ci.group.size = 1, alpha = alpha, imbalance.penalty = imbalance.penalty,
                                     clusters = clusters, samples_per_cluster = samples_per_cluster);
       Z.hat <- predict(forest.Z)$predictions
     } else if (length(Z.hat) == 1) {
@@ -127,9 +135,9 @@ instrumental_forest <- function(X, Y, W, Z,
     
     forest <- instrumental_train(data$default, data$sparse, outcome.index, treatment.index,
         instrument.index, mtry, num.trees, num.threads, min.node.size, sample.fraction, seed, honesty,
-        ci.group.size, reduced.form.weight, alpha, imbalance.penalty, stabilize.splits,
-        clusters, samples_per_cluster)
-    
+        coerce_honesty_fraction(honesty.fraction), ci.group.size, reduced.form.weight, alpha, 
+        imbalance.penalty, stabilize.splits, clusters, samples_per_cluster)
+
     forest[["ci.group.size"]] <- ci.group.size
     forest[["X.orig"]] <- X
     forest[["Y.orig"]] <- Y
@@ -139,8 +147,15 @@ instrumental_forest <- function(X, Y, W, Z,
     forest[["W.hat"]] <- W.hat
     forest[["Z.hat"]] <- Z.hat
     forest[["clusters"]] <- clusters
-    
+
     class(forest) <- c("instrumental_forest", "grf")
+
+    if (compute.oob.predictions) {
+        oob.pred <- predict(forest)
+        forest[["predictions"]] <- oob.pred$predictions
+        forest[["debiased.error"]] <- oob.pred$debiased.error
+    }
+
     forest
 }
 
@@ -160,27 +175,41 @@ instrumental_forest <- function(X, Y, W, Z,
 #' @param ... Additional arguments (currently ignored).
 #'
 #' @return Vector of predictions, along with (optional) variance estimates.
+#'
+#' @method predict instrumental_forest
 #' @export
 predict.instrumental_forest <- function(object, newdata = NULL,
                                         num.threads = NULL, 
                                         estimate.variance = FALSE,
                                         ...) {
-    num.threads <- validate_num_threads(num.threads)    
-    if (estimate.variance) {
-        ci.group.size = object$ci.group.size
-    } else {
-        ci.group.size = 1
+
+    # If possible, use pre-computed predictions.
+    if (is.null(newdata) & !estimate.variance & !is.null(object$predictions)) {
+        return(data.frame(predictions=object$predictions,
+                          debiased.error=object$debiased.error))
     }
-    
+
+    num.threads <- validate_num_threads(num.threads)
+
+    if (estimate.variance) {
+        ci.group.size <- object$ci.group.size
+    } else {
+        ci.group.size <- 1
+    }
+
     forest.short <- object[-which(names(object) == "X.orig")]
-    
+
     if (!is.null(newdata)) {
         data <- create_data_matrices(newdata)
-        instrumental_predict(forest.short, data$default, data$sparse,
-                             num.threads, ci.group.size)
+        ret <- instrumental_predict(forest.short, data$default, data$sparse,
+                                    num.threads, ci.group.size)
     } else {
         data <- create_data_matrices(object[["X.orig"]])
-        instrumental_predict_oob(forest.short, data$default, data$sparse,
-                                 num.threads, ci.group.size)
+        ret <- instrumental_predict_oob(forest.short, data$default, data$sparse,
+                                        num.threads, ci.group.size)
     }
+
+    # Convert list to data frame.
+    empty = sapply(ret, function(elem) length(elem) == 0)
+    do.call(cbind.data.frame, ret[!empty])
 }
