@@ -22,6 +22,7 @@ Rcpp::List quantile_train(std::vector<double> quantiles,
                           double sample_fraction,
                           unsigned int seed,
                           bool honesty,
+                          double honesty_fraction,
                           unsigned int ci_group_size,
                           double alpha,
                           double imbalance_penalty,
@@ -32,8 +33,8 @@ Rcpp::List quantile_train(std::vector<double> quantiles,
       : ForestTrainers::quantile_trainer(outcome_index - 1, quantiles);
 
   Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
-  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size,
-      honesty, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
+  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size, honesty,
+      honesty_fraction, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
 
   Forest forest = trainer.train(data, options);
 
