@@ -20,6 +20,7 @@ Rcpp::List regression_train(Rcpp::NumericMatrix input_data,
                             double sample_fraction,
                             unsigned int seed,
                             bool honesty,
+                            double honesty_fraction,
                             unsigned int ci_group_size,
                             double alpha,
                             double imbalance_penalty,
@@ -28,8 +29,8 @@ Rcpp::List regression_train(Rcpp::NumericMatrix input_data,
   ForestTrainer trainer = ForestTrainers::regression_trainer(outcome_index - 1);
 
   Data* data = RcppUtilities::convert_data(input_data, sparse_input_data);
-  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size,
-      honesty, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
+  ForestOptions options(num_trees, ci_group_size, sample_fraction, mtry, min_node_size, honesty,
+      honesty_fraction, alpha, imbalance_penalty, num_threads, seed, clusters, samples_per_cluster);
 
   Forest forest = trainer.train(data, options);
 
@@ -82,7 +83,7 @@ Rcpp::List local_linear_predict(Rcpp::List forest,
                                 Rcpp::NumericMatrix training_data,
                                 Eigen::SparseMatrix<double> sparse_input_data,
                                 Eigen::SparseMatrix<double> sparse_training_data,
-                                double lambda,
+                                std::vector<double> lambdas,
                                 bool use_unweighted_penalty,
                                 std::vector<size_t> linear_correction_variables,
                                 unsigned int num_threads) {
@@ -92,7 +93,7 @@ Rcpp::List local_linear_predict(Rcpp::List forest,
   Forest deserialized_forest = RcppUtilities::deserialize_forest(forest[RcppUtilities::SERIALIZED_FOREST_KEY]);
 
   ForestPredictor predictor = ForestPredictors::local_linear_predictor(num_threads, original_data, test_data,
-                                                                       lambda, use_unweighted_penalty,
+                                                                       lambdas, use_unweighted_penalty,
                                                                        linear_correction_variables);
   std::vector<Prediction> predictions = predictor.predict(deserialized_forest, test_data);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
@@ -106,7 +107,7 @@ Rcpp::List local_linear_predict(Rcpp::List forest,
 Rcpp::List local_linear_predict_oob(Rcpp::List forest,
                                     Rcpp::NumericMatrix input_data,
                                     Eigen::SparseMatrix<double> sparse_input_data,
-                                    double lambda,
+                                    std::vector<double> lambdas,
                                     bool use_unweighted_penalty,
                                     std::vector<size_t> linear_correction_variables,
                                     unsigned int num_threads) {
@@ -115,7 +116,7 @@ Rcpp::List local_linear_predict_oob(Rcpp::List forest,
   Forest deserialized_forest = RcppUtilities::deserialize_forest(forest[RcppUtilities::SERIALIZED_FOREST_KEY]);
 
   ForestPredictor predictor = ForestPredictors::local_linear_predictor(num_threads, data, data,
-                                                                       lambda, use_unweighted_penalty,
+                                                                       lambdas, use_unweighted_penalty,
                                                                        linear_correction_variables);
   std::vector<Prediction> predictions = predictor.predict_oob(deserialized_forest, data);
   Rcpp::List result = RcppUtilities::create_prediction_object(predictions);
