@@ -25,6 +25,7 @@
 #include "tree/Tree.h"
 #include "tree/TreeTrainer.h"
 #include "forest/Forest.h"
+#include "ForestOptions.h"
 
 #include <future>
 #include <memory>
@@ -33,70 +34,32 @@
 
 class ForestTrainer {
 public:
-  ForestTrainer(std::unordered_map<size_t, size_t> observables,
+  ForestTrainer(const std::unordered_map<size_t, size_t>& observables,
                 std::shared_ptr<RelabelingStrategy> relabeling_strategy,
                 std::shared_ptr<SplittingRuleFactory> splitting_rule_factory,
                 std::shared_ptr<OptimizedPredictionStrategy> prediction_strategy);
-
-  Forest train(Data* data);
-
-  void init(uint mtry,
-            uint num_trees,
-            uint seed,
-            uint num_threads,
-            uint min_node_size,
-            std::set<size_t> no_split_variables,
-            std::string split_select_weights_file,
-            bool sample_with_replacement,
-            std::string case_weights_file,
-            double sample_fraction,
-            bool honesty,
-            uint ci_group_size);
+  const Forest train(Data* data, const ForestOptions& options) const;
 
 private:
+
+  std::vector<std::shared_ptr<Tree>> train_batch(
+      size_t start, size_t num_trees,
+      Data* data,
+      const Observations& observations,
+      const ForestOptions& options) const;
+
+  std::shared_ptr<Tree> train_tree(Data* data,
+                                   const Observations& observations,
+                                   RandomSampler& sampler,
+                                   const ForestOptions& options) const;
+
   std::vector<std::shared_ptr<Tree>> train_ci_group(Data* data,
                                                     const Observations& observations,
                                                     RandomSampler& sampler,
-                                                    double sample_fraction);
-
-  std::vector<std::shared_ptr<Tree>> train_batch(
-      size_t thread_index,
-      size_t num_trees,
-      Data* data,
-      const Observations& observations);
-
-  void set_split_select_weights(std::vector<double>& split_select_weights,
-                                size_t num_independent_variables);
-
-  size_t num_trees;
-  uint mtry;
-  uint min_node_size;
-  uint seed;
-  bool sample_with_replacement;
-  double sample_fraction;
-
-  std::set<size_t> no_split_variables;
-
-  uint num_threads;
-  std::shared_ptr<TreeTrainer> tree_trainer;
-
-  // Weight vector for selecting possible split variables, one weight between 0 (never select) and 1 (always select) for each variable
-  // Deterministic variables are always selected
-  std::vector<size_t> deterministic_vars;
-  std::vector<size_t> split_select_vars;
-  std::vector<double> split_select_weights;
-
+                                                    const ForestOptions& options) const;
 
   std::unordered_map<size_t, size_t> observables;
-  std::shared_ptr<RelabelingStrategy> relabeling_strategy;
-  std::shared_ptr<SplittingRuleFactory> splitting_rule_factory;
-  std::shared_ptr<OptimizedPredictionStrategy> prediction_strategy;
-
-  std::vector<double> sample_weights;
-  std::string sample_weights_file;
-  std::string split_select_weights_file;
-
-  uint ci_group_size;
+  TreeTrainer tree_trainer;
 };
 
 
