@@ -31,11 +31,19 @@ class LocalLinearPredictionStrategy: public DefaultPredictionStrategy {
 public:
     LocalLinearPredictionStrategy(const Data *original_data,
                                   const Data *test_data,
-                                  double lambda,
+                                  std::vector<double> lambdas,
                                   bool use_unweighted_penalty,
                                   std::vector<size_t> linear_correction_variables);
 
     size_t prediction_length();
+
+    /**
+    * LocalLinearPredictionStrategy::predict computes a regularization path.
+    *
+    * Lambdas is a set of potential regularization parameters, and the forest will
+    *   output predictions along each of these parameters.
+    */
+
     std::vector<double> predict(size_t sampleID,
                                 const std::unordered_map<size_t, double>& weights_by_sampleID,
                                 const Observations& observations);
@@ -44,7 +52,7 @@ private:
     static const std::size_t OUTCOME;
     const Data *original_data;
     const Data *test_data;
-    double lambda;
+    std::vector<double> lambdas;
     bool use_unweighted_penalty;
     std::vector<size_t> linear_correction_variables;
 };
