@@ -35,16 +35,36 @@ public:
        const std::vector<std::vector<size_t>>& leaf_samples,
        const std::vector<size_t>& split_vars,
        const std::vector<double>& split_values,
-       const std::vector<size_t>& oob_samples,
+       const std::vector<size_t>& drawn_samples,
        const PredictionValues& prediction_values);
 
   /**
    * Given test data and a list of sample IDs, recurses down the tree to find
    * the leaf node IDs that those samples belong in.
+   *
+   * @param prediction_data: the data matrix containing all test samples.
+   * @param samples: a list of sample IDs whose leaf nodes should be calculated.
+   * @return The resulting node IDs for each sample ID in the input. For efficiency reasons, this
+   * vector's length will be equal to the total number of test samples, and the index for each
+   * requested sample ID will contain the corresponding node ID. All other values will be 0.
    */
   std::vector<size_t> find_leaf_nodes(Data* prediction_data,
                                       const std::vector<size_t>& samples);
 
+  /**
+   * Given test data and a vector indicating which samples to consider, recurses
+   * down the tree to find the leaf node IDs that those samples belong in.
+   *
+   * @param prediction_data: the data matrix containing all test samples.
+   * @param samples: a vector indicating which samples should have their leaf nodes
+   * calculated. This vector's length should be equal to the total number of samples, and
+   * contain 'true' for a sample ID's index if that sample's leaf node should be calculated.
+   * @return The resulting node IDs for each sample ID in the input. For efficiency reasons, this
+   * vector's length will be equal to the total number of test samples, and the index for each
+   * requested sample ID will contain the corresponding node ID. All other values will be 0.
+   */
+  std::vector<size_t> find_leaf_nodes(Data* prediction_data,
+                                      const std::vector<bool>& valid_samples);
   /**
    * Removes all empty leaf nodes.
    *
@@ -58,73 +78,65 @@ public:
    * The ID of the root node for this tree. Note that this is usually 0, but may not always
    * be as the top of the tree can be pruned.
    */
-  size_t get_root_node() {
-    return root_node;
-  }
+  size_t get_root_node();
 
   /**
    * A vector containing two vectors: the first gives the ID of the left child for every
    * node, and the second gives the ID of the right child. If a node is a leaf, the entries
    * for both the left and right children will be '0'.
    */
-  const std::vector<std::vector<size_t>>& get_child_nodes() {
-    return child_nodes;
-  }
+  const std::vector<std::vector<size_t>>& get_child_nodes();
 
   /**
    * Specifies the samples that each node contains. Note that only leaf nodes will contain
    * a non-empty vector of sample IDs.
    */
-  const std::vector<std::vector<size_t>>& get_leaf_samples() {
-    return leaf_samples;
-  }
+  const std::vector<std::vector<size_t>>& get_leaf_samples();
 
   /**
    * For each split, the ID of the variable that was chosen to split on.
    */
-  const std::vector<size_t>& get_split_vars() {
-    return split_vars;
-  }
+  const std::vector<size_t>& get_split_vars();
 
   /**
    * For each split, the value of the variable that was chosen to split on.
    */
-  const std::vector<double>& get_split_values() {
-    return split_values;
-  }
+  const std::vector<double>& get_split_values();
 
   /**
    * The sample IDs that were not drawn in creating this tree. For honest trees,
    * this excludes both samples that went into growing the tree, as well as samples
    * used to repopulate the leaves.
    */
-  const std::vector<size_t>& get_oob_samples() {
-    return oob_samples;
-  }
+  const std::vector<size_t>& get_drawn_samples();
 
   /**
    * Optional summary values about the samples in each leaf. Note that this will only
    * be non-empty if the tree was trained with an 'optimized' prediction strategy.
    */
-  const PredictionValues& get_prediction_values() {
-    return prediction_values;
-  }
+  const PredictionValues& get_prediction_values();
 
+  /**
+   * Given a node ID, returns true if the node represents a leaf in this tree (in
+   * particular, the node has no children).
+   */
   bool is_leaf(size_t node);
 
-  void set_leaf_nodes(const std::vector<std::vector<size_t>>& leaf_nodes) {
-    this->leaf_samples = leaf_nodes;
-  }
+  /**
+   * Sets the contents of this tree's leaf nodes. Please see
+   * Tree::get_leaf_samples for a description of this variable.
+   */
+  void set_leaf_samples(const std::vector<std::vector<size_t>>& leaf_samples);
 
-  void set_oob_samples(const std::vector<size_t> &oob_samples) {
-    this->oob_samples = oob_samples;
-  }
-
-  void set_prediction_values(const PredictionValues& prediction_values) {
-    this->prediction_values = prediction_values;
-  }
+  /**
+   * Sets the contents of this tree's prediction values. Please see
+   * Tree::get_prediction_values for a description of this variable.
+   */
+  void set_prediction_values(const PredictionValues& prediction_values);
 
 private:
+  size_t find_leaf_node(Data* prediction_data,
+                        size_t sample);
   void prune_node(size_t& node);
   bool is_empty_leaf(size_t node);
 
@@ -134,7 +146,7 @@ private:
   std::vector<size_t> split_vars;
   std::vector<double> split_values;
 
-  std::vector<size_t> oob_samples;
+  std::vector<size_t> drawn_samples;
 
   PredictionValues prediction_values;
 };
